/* ---------------------------------------------------------------------------

 DCPhoenix
 Version 0.4

 Chris White (pointblnk@hotmail.com)
 http://www.alienwebshop.com/pointblank/dc/

 * Based on Jabawape 2.1 Source Code (D. Finck & Th. Strathmann)

 You are free to do what you like with this code, but please read the
 licence for the Z80 emulator in the relevant subdirectory.

 If you're a DC coder, then I'd encourage you to release your source code.
 We're not going to make any money by keeping it hidden, and we're not going
 to lose anything by releasing it! (apart from helping others).

----------------------------------------------------------------------------*/

#include "dream.h"				/* DreamLib 0.95 */
#include "Z80/Z80.h"				/* Include Z80A CPU emulator */

#include "phoenix.h"
#include "memory.h"
#include "video.h"
#include "sound.h"				/* AICA Firmware and sample support */
#include "vmulcd.h"				/* Include VMU LCD routines */

/* ---------------------------------------------------------------------------
   Main Method
 ----------------------------------------------------------------------------*/

int dc_main(void)
{
	/* -- Setup Naughty Global Variables ----- */

	VRAM = 0;				/* Current VRAM bank */
	pallete = 1;				/* Select Pallete 1 */
	skip = 0;				/* Set number of skipped frames */
	frameskip = 1;				/* Set Frameskip */
	fskipmax = 3;				/* Used if we enable adjusting of
						   frames within the emulator */
	Dips = 0x41;               		/* settings of hardware dips (4 Ships) */

	serial_disable();			/* Disable the serial port */
	dc_setup(DM_640x480, PM_RGB565);	/* Set Graphics Mode 640x480x16 */

	display_info();				/* Display welcome information */

	clearscr(640, 480);			/* Clear Screen */

	/* void bfont_draw_str(uint16 *buffer, int bufwidth, int c) */
	/* eg vram_s+Ycordinate*320+Xcordinate, 320, "blah"); */
	bfont_draw_str(vram_s+20*640+20, 640, "DCPhoenix");

	if (AllocateMem())			/* Setup Memory */
		return (1);

	read_roms();				/* Read Game ROMs into Memory */

	bfont_draw_str(vram_s+180*640+20, 640, "Resetting Z80 . . .");
	ResetZ80(&R);				/* Initialise CPU */
	bfont_draw_str(vram_s+210*640+20, 640, "Starting Z80  . . .");

	sleep(1000);				/* Pause for one second */

	clearscr(640, 480);
	dc_setup(DM_320x240, PM_RGB565);	/* Switch to 320x240 */

	if (game == 1)
		sound_setup();			/* Initialise Sound */

	Z80(R);					/* Start Z80 CPU */
	dc_setup(DM_640x480, PM_RGB565);
	clearscr(640, 480);

	bfont_draw_str(vram_s+20*640+20, 640, "Goodbye!");

	//free(RAM);				/* Free Up Memory */

	return (0);
}

/* ---------------------------------------------------------------------------
   Display Welcoming Screen to User
 ----------------------------------------------------------------------------*/

void display_info(void)
{
	uint8 mcont, mvmu;
	cont_cond_t cond;
	int phase = 0;				/* For VMU Animation Cycling */

	mcont = maple_controller_addr();	/* Get Controller Address */
	mvmu = maple_lcd_addr();		/* Get LCD Address */

	clearscr(640, 480);

	if (!mcont)
	{
		bfont_draw_str(vram_s+20*640+20, 640, "Please plug in a controller!");
		while(!mcont){}
		return;
	}

	else if (cont_get_cond(mcont, &cond))
	{
		bfont_draw_str(vram_s+50*640+20, 640, "Error getting controller status.");
		while(cont_get_cond(mcont, &cond)){}
		return;
	}

	else
	{
		bfont_draw_str(vram_s+20*640+20, 640, "DCPhoenix Emulator 0.4");
		bfont_draw_str(vram_s+50*640+20, 640, "pointblnk@hotmail.com");

		bfont_draw_str(vram_s+90*640+20, 640, "www.alienwebshop.com/pointblank/dc/");

		bfont_draw_str(vram_s+130*640+20, 640, "Controls are:");
		bfont_draw_str(vram_s+160*640+20, 640, "D-PAD        Movement");
		bfont_draw_str(vram_s+185*640+20, 640, "A            Fire");
		bfont_draw_str(vram_s+210*640+20, 640, "B            Shield (Phoenix Only)");

		bfont_draw_str(vram_s+240*640+20, 640, "X            Insert Coin");
		bfont_draw_str(vram_s+265*640+20, 640, "L Trigger    1P Start");
		bfont_draw_str(vram_s+290*640+20, 640, "R Trigger    2P Start");
		bfont_draw_str(vram_s+315*640+20, 640, "START        Return To Menu");

		bfont_draw_str(vram_s+380*640+20, 640, "Press A to play Phoenix");
		bfont_draw_str(vram_s+410*640+20, 640, "Press B to play Pleiads (No Sound)");


		while(1) /* Wait for user to select a game */
		{

	    		if(cont_get_cond(mcont, &cond))
	      		{
				return;
		    	}


	      		if(!(cond.buttons  & CONT_A))
			{
				game = 1;
				if (mvmu) vmu_lcd_update(1, mvmu); /* Stick main image on LCD */
				break;
		 	}

	      		if(!(cond.buttons  & CONT_B))
			{
				game = 2;
				if (mvmu) vmu_lcd_update(1, mvmu);
				break;
		 	}

		 	if(mvmu)				/* If there's a VMU attached */
		 	{
				if (phase > 4) phase = 1;	/* Reset to first image */
				vmu_lcd_update(phase, mvmu);	/* Stick Image on VMU */
				phase++;			/* Increment phase of animation */
			}

		 	sleep(100);
		}
	}
}

/*** Phoenix Hardware Specification ***
Resolution 26x8 = 208 columns x 32x8 = 256 lines

Phoenix memory map
  0000-3fff 16Kb Program ROM
  4000-43ff 1Kb Video RAM Charset A (4340-43ff variables)
  4400-47ff 1Kb Work RAM
  4800-4bff 1Kb Video RAM Charset B (4840-4bff variables)
  4c00-4fff 1Kb Work RAM
  5000-53ff 1Kb Video Control write-only (mirrored)
  5400-47ff 1Kb Work RAM
  5800-5bff 1Kb Video Scroll Register (mirrored)
  5c00-5fff 1Kb Work RAM
  6000-63ff 1Kb Sound Control A (mirrored)
  6400-67ff 1Kb Work RAM
  6800-6bff 1Kb Sound Control B (mirrored)
  6c00-6fff 1Kb Work RAM
  7000-73ff 1Kb 8bit Game Control read-only (mirrored)
  7400-77ff 1Kb Work RAM
  7800-7bff 1Kb 8bit Dip Switch read-only (mirrored)
  7c00-7fff 1Kb Work RAM

  memory mapped ports:

  read-only:
  7000-73ff IN
  7800-7bff DSW

		* IN (all bits are inverted)
		* bit 7 : barrier
		* bit 6 : Left
		* bit 5 : Right
		* bit 4 : Fire
		* bit 3 : -
		* bit 2 : Start 2
		* bit 1 : Start 1
		* bit 0 : Coin

	    * DSW
 	    * bit 7 : VBlank
	    * bit 6 : free play (pleiads only)
	    * bit 5 : attract sound 0 = off 1 = on (pleiads only?)
	    * bit 4 : coins per play	0 = 1 coin	1 = 2 coins
	    * bit 3 :\ bonus
	    * bit 2 :/ 00 = 3000	01 = 4000  10 = 5000  11 = 6000
	    * bit 1 :\ number of lives
	    * bit 0 :/ 00 = 3  01 = 4  10 = 5  11 = 6

		Pallete
		//0 	]
		//1 	] bit 5-7 of video ram value (divides 256 chars in 8 color sections)
		//2 	]
		//3 ] 2 bit pixelcolor
		//4 ] (either from CHAR-A or CHAR-B, depends on Bit5)
		//5 0= CHAR-A, 1= CHAR-B
		//6 palette flag (see video control reg.)
		//7 always 0
*/
